/*
 * Copyright (c) 2019 Hemanth Savarala.
 *
 * Licensed under the GNU General Public License v3
 *
 * This is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by
 *  the Free Software Foundation either version 3 of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 */

package code.name.monkey.retromusic.util

import android.animation.*
import android.content.Context
import android.content.res.*
import android.graphics.*
import android.graphics.drawable.LayerDrawable
import android.os.Build
import android.view.View
import android.view.animation.PathInterpolator
import android.widget.*
import androidx.annotation.ColorInt
import androidx.core.view.ViewCompat
import code.name.monkey.appthemehelper.ThemeStore
import code.name.monkey.appthemehelper.util.*
import code.name.monkey.retromusic.R
import com.simplecityapps.recyclerview_fastscroll.views.FastScrollRecyclerView

object ViewUtil {

	const val RETRO_MUSIC_ANIM_TIME = 1000

	fun setProgressDrawable(progressSlider: SeekBar, newColor: Int, thumbTint: Boolean = false) {

		if (thumbTint) {
			progressSlider.thumbTintList = ColorStateList.valueOf(newColor)
		}
		if (Build.VERSION.SDK_INT <= Build.VERSION_CODES.LOLLIPOP_MR1) {
			val layerDrawable = progressSlider.progressDrawable as LayerDrawable
			val progressDrawable = layerDrawable.findDrawableByLayerId(android.R.id.progress)
			progressDrawable.setColorFilter(newColor, PorterDuff.Mode.SRC_IN)
		} else {
			progressSlider.progressTintList = ColorStateList.valueOf(newColor)
		}
	}

	fun setProgressDrawable(progressSlider: ProgressBar, newColor: Int) {

		val ld = progressSlider.progressDrawable as LayerDrawable

		val progress = ld.findDrawableByLayerId(android.R.id.progress)
		progress.setColorFilter(newColor, PorterDuff.Mode.SRC_IN)

		val background = ld.findDrawableByLayerId(android.R.id.background)
		val primaryColor = ATHUtil.resolveColor(progressSlider.context, R.attr.colorPrimary)
		background.setColorFilter(
				MaterialValueHelper.getPrimaryDisabledTextColor(
						progressSlider.context, ColorUtil.isColorLight(
						primaryColor
				)
				), PorterDuff.Mode.SRC_IN
		)

		val secondaryProgress = ld.findDrawableByLayerId(android.R.id.secondaryProgress)
		secondaryProgress?.setColorFilter(
				ColorUtil.withAlpha(newColor, 0.65f), PorterDuff.Mode.SRC_IN
		)
	}

	private fun createColorAnimator(
			target: Any, propertyName: String, @ColorInt startColor: Int, @ColorInt endColor: Int
	): Animator {
		val animator: ObjectAnimator
		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
			animator = ObjectAnimator.ofArgb(target, propertyName, startColor, endColor)
		} else {
			animator = ObjectAnimator.ofInt(target, propertyName, startColor, endColor)
			animator.setEvaluator(ArgbEvaluator())
		}

		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
			animator.interpolator = PathInterpolator(0.4f, 0f, 1f, 1f)
		}
		animator.duration = RETRO_MUSIC_ANIM_TIME.toLong()
		return animator
	}

	fun hitTest(v: View, x: Int, y: Int): Boolean {
		val tx = (ViewCompat.getTranslationX(v) + 0.5f).toInt()
		val ty = (ViewCompat.getTranslationY(v) + 0.5f).toInt()
		val left = v.left + tx
		val right = v.right + tx
		val top = v.top + ty
		val bottom = v.bottom + ty

		return x in left..right && y >= top && y <= bottom
	}

	fun setUpFastScrollRecyclerViewColor(
			context: Context,
			recyclerView: FastScrollRecyclerView,
			accentColor: Int = ThemeStore.accentColor(context)
	) {
		recyclerView.setPopupBgColor(accentColor)
		recyclerView.setPopupTextColor(
				MaterialValueHelper.getPrimaryTextColor(
						context, ColorUtil.isColorLight(
						accentColor
				)
				)
		)
		recyclerView.setThumbColor(accentColor)
		recyclerView.setTrackColor(Color.TRANSPARENT)
		recyclerView.setTrackColor(
				ColorUtil.withAlpha(
						ATHUtil.resolveColor(
								context, R.attr.colorControlNormal
						), 0.12f
				)
		)

	}

	fun convertDpToPixel(dp: Float, resources: Resources): Float {
		val metrics = resources.displayMetrics
		return dp * metrics.density
	}
}